//+------------------------------------------------------------------+
//|                                                   TimeFrames.mqh |
//|                                                      nicholishen |
//|                                   www.reddit.com/u/nicholishenFX |
//+------------------------------------------------------------------+
#property copyright "nicholishen"
#property link      "http://www.reddit.com/u/nicholishenFX"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+

#define BOOL(f,x) bool(f&(1<<x))
#define FLAG(x) (1<<x)
#define NUM_TFS 22

#include <Arrays\ArrayObj.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum TF_FLAGS
{
   FLAG_NO_TF  =0,            FLAG_M1     =0x00000001,   FLAG_M2     =0x00000002,
   FLAG_M3     =0x00000004,   FLAG_M4     =0x00000008,   FLAG_M5     =0x00000010,
   FLAG_M6     =0x00000020,   FLAG_M10    =0x00000040,   FLAG_M12    =0x00000080,
   FLAG_M15    =0x00000100,   FLAG_M20    =0x00000200,   FLAG_M30    =0x00000400,
   FLAG_H1     =0x00000800,   FLAG_H2     =0x00001000,   FLAG_H3     =0x00002000,
   FLAG_H4     =0x00004000,   FLAG_H6     =0x00008000,   FLAG_H8     =0x00010000,
   FLAG_H12    =0x00020000,   FLAG_D1     =0x00040000,   FLAG_W1     =0x00080000,
   FLAG_MN1    =0x00100000,   FLAG_ALL_TF =0x001fffff
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

class CTF : public CObject
{
protected:
   ENUM_TIMEFRAMES   m_period;
public:  
                     CTF():m_period(NULL){}
                     CTF(const ENUM_TIMEFRAMES period):m_period(period){}
   string            ToString()     { return EnumToString(m_period);}
   ENUM_TIMEFRAMES   Period() const { return m_period;}
   int               Compare(const CObject *node,const int mode=0)const
                     {
                        CTF *other = (CTF*)node;
                        if(this.Period() > other.Period()) return 1;
                        if(this.Period() < other.Period()) return -1;
                        return 0;
                     }      
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTimeFrames
{
protected:
   int               m_selected;
   CArrayObj         m_arr;
public:
                     CTimeFrames();
                     CTimeFrames(const int flags);
   void              SetPeriodsByFlags(const int flags);
   void              SetPeriodsByIterator(const int iter);
   string            AllToString();
   ENUM_TIMEFRAMES   Period();
   int               Total() const { return m_arr.Total(); }
   int               ToSeconds();
   string            ToString();
   void              GetFirst() { m_selected = 0;}
   void              GetLast()  { m_selected = Total()-1;}
   bool              First()  { return m_selected<0?true:false;}
   bool              Last()   { return m_selected>Total()-1?true:false;}
   void              Next()   { m_selected++;}
   void              Prev()   { m_selected--;} 
   CTF*              operator[] (const int index) const { return (CTF*)m_arr.At(index);}
   int               operator = (const ENUM_TIMEFRAMES period);
   int               operator+= (const ENUM_TIMEFRAMES period) { return this = period; }
   bool              operator <=(const ENUM_TIMEFRAMES period);
   bool              operator >=(const ENUM_TIMEFRAMES period);
   void              operator++(int){ m_selected++;}
   void              operator--(int){ m_selected--;}  
protected:
   ENUM_TIMEFRAMES   FlagToEnum(int flag);
};
//+------------------------------------------------------------------+
CTimeFrames::CTimeFrames(const int flags):m_selected(0)
{
   SetPeriodsByFlags(flags);   
};
//+------------------------------------------------------------------+
CTimeFrames::CTimeFrames(void):m_selected(0)
{
   SetPeriodsByFlags(OBJ_ALL_PERIODS);   
};
//+------------------------------------------------------------------+
ENUM_TIMEFRAMES CTimeFrames::Period(void)
{
   if(m_arr.Total()>0 && m_selected >=0 && m_selected < m_arr.Total())
      return this[m_selected].Period();
   return PERIOD_CURRENT;
}
//+------------------------------------------------------------------+
int CTimeFrames::operator = (const ENUM_TIMEFRAMES period)
{
   for(int i=0;i<m_arr.Total();i++)
   {
      if(period == this[i].Period())
      {
         m_selected = i;
         return i;
      }
   }
   m_arr.Sort();
   m_arr.InsertSort(new CTF(period));
   return this = period; ///recursive method call
}
//+------------------------------------------------------------------+
bool CTimeFrames::operator <=(const ENUM_TIMEFRAMES period)
{
   if(this.Period()<= period && Period() != PERIOD_CURRENT)
      return true;
   return false;
}
//+------------------------------------------------------------------+
bool CTimeFrames::operator >=(const ENUM_TIMEFRAMES period)
{
   if(this.Period()>= period && Period() != PERIOD_CURRENT)
      return true;
   return false;
}

//+------------------------------------------------------------------+
void CTimeFrames::SetPeriodsByIterator(const int iter)
{
   SetPeriodsByFlags(1<<iter);
}
//+------------------------------------------------------------------+
void CTimeFrames::SetPeriodsByFlags(const int flags)
{
   m_arr.Clear();
   for(int i=0;i<NUM_TFS;i++)
      if(BOOL(flags,i))
         m_arr.Add(new CTF(FlagToEnum(FLAG(i))));
}
//+------------------------------------------------------------------+
string CTimeFrames::AllToString(void)
{
   string res="";
   for(int i=0;i<m_arr.Total();i++)
      res+= this[i].ToString()+ (i==m_arr.Total()-1?"":", ");
   return res;
}
//+------------------------------------------------------------------+
ENUM_TIMEFRAMES CTimeFrames::FlagToEnum(int flag)
{
   if(flag == FLAG_NO_TF) return PERIOD_CURRENT;
   if(flag == FLAG_M1)  return PERIOD_M1;
   if(flag == FLAG_M2)  return PERIOD_M2;
   if(flag == FLAG_M3)  return PERIOD_M3;
   if(flag == FLAG_M4)  return PERIOD_M4;
   if(flag == FLAG_M5)  return PERIOD_M5;
   if(flag == FLAG_M6)  return PERIOD_M6;
   if(flag == FLAG_M10) return PERIOD_M10;
   if(flag == FLAG_M12) return PERIOD_M12;
   if(flag == FLAG_M15) return PERIOD_M15;
   if(flag == FLAG_M20) return PERIOD_M20;
   if(flag == FLAG_M30) return PERIOD_M30;
   if(flag == FLAG_H1)  return PERIOD_H1;
   if(flag == FLAG_H2)  return PERIOD_H2;
   if(flag == FLAG_H3)  return PERIOD_H3;
   if(flag == FLAG_H4)  return PERIOD_H4;
   if(flag == FLAG_H6)  return PERIOD_H6;
   if(flag == FLAG_H8)  return PERIOD_H8;
   if(flag == FLAG_H12) return PERIOD_H12;
   if(flag == FLAG_D1)  return PERIOD_D1;
   if(flag == FLAG_W1)  return PERIOD_W1;
   if(flag == FLAG_MN1) return PERIOD_MN1;
   return PERIOD_CURRENT;
}
//+------------------------------------------------------------------+
string CTimeFrames::ToString(void)
{
   return EnumToString(this.Period());
}
//+------------------------------------------------------------------+
int CTimeFrames::ToSeconds(void)
{
   return PeriodSeconds(this.Period());
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
